<div class="btn-group">
    <button type="button" class="btn btn-default dropdown-toggle" data-toggle="dropdown">
        <span ng-if="ctxtObj.current">[[ctxtObj.name]]: [[ctxtObj.current]]</span>
        <span ng-if="!ctxtObj.current">[[ctxtObj.name]]</span>
        <span class="caret"></span>
    </button>
    <ul class="dropdown-menu" role="menu">
        <li ng-repeat="value in ctxtObj.values">
            <a ng-href="[[ctxtObj.getUrl(value)]]">[[value]]</a>
        </li>
    </ul>
</div>
