<div class="modal fade" id="[[modalId]]" tabindex="-1" role="dialog">
    <div class="modal-dialog" ng-class="{
        'modal-lg': modalSize == 'large',
        'modal-sm': modalSize == 'small'
    }">
        <div class="modal-content">
            <div class="modal-header">
                <button type="button" class="close close-light" data-dismiss="modal">
                    <span class="fa fa-close"</span>
                </button>
                <h4 class="modal-title">[[title]]</h4>
            </div>
            <ng-transclude></ng-transclude>
        </div>
    </div>
</div>

